#!/usr/bin/env node
import { Server } from "@modelcontextprotocol/sdk/server/index.js";
import { StdioServerTransport } from "@modelcontextprotocol/sdk/server/stdio.js";
import { CallToolRequestSchema, ListToolsRequestSchema, ToolSchema, } from "@modelcontextprotocol/sdk/types.js";
import fs from "fs/promises";
import path from "path";
import os from 'os';
import { z } from "zod";
import { zodToJsonSchema } from "zod-to-json-schema";
import { minimatch } from 'minimatch';
import { XliffProcessor } from './xliff-processor.js';
// 命令行参数解析
const args = process.argv.slice(2);
if (args.length === 0) {
    console.error("使用方法: xliff-mcp <允许的目录> [其他目录...]");
    process.exit(1);
}
// 规范化路径
function normalizePath(p) {
    return path.normalize(p);
}
function expandHome(filepath) {
    if (filepath.startsWith('~/') || filepath === '~') {
        return path.join(os.homedir(), filepath.slice(1));
    }
    return filepath;
}
// 存储允许的目录
const allowedDirectories = args.map(dir => normalizePath(path.resolve(expandHome(dir))));
// 验证所有目录是否存在且可访问
await Promise.all(args.map(async (dir) => {
    try {
        const stats = await fs.stat(expandHome(dir));
        if (!stats.isDirectory()) {
            console.error(`错误: ${dir} 不是一个目录`);
            process.exit(1);
        }
    }
    catch (error) {
        console.error(`访问目录 ${dir} 时出错:`, error);
        process.exit(1);
    }
}));
// 安全工具函数
async function validatePath(requestedPath) {
    const expandedPath = expandHome(requestedPath);
    const absolute = path.isAbsolute(expandedPath)
        ? path.resolve(expandedPath)
        : path.resolve(process.cwd(), expandedPath);
    const normalizedRequested = normalizePath(absolute);
    // 检查路径是否在允许的目录中
    const isAllowed = allowedDirectories.some(dir => normalizedRequested.startsWith(dir));
    if (!isAllowed) {
        throw new Error(`访问被拒绝 - 路径在允许的目录之外: ${absolute}`);
    }
    try {
        const realPath = await fs.realpath(absolute);
        const normalizedReal = normalizePath(realPath);
        const isRealPathAllowed = allowedDirectories.some(dir => normalizedReal.startsWith(dir));
        if (!isRealPathAllowed) {
            throw new Error("访问被拒绝 - 符号链接目标在允许的目录之外");
        }
        return realPath;
    }
    catch (error) {
        // 对于尚不存在的新文件，验证父目录
        const parentDir = path.dirname(absolute);
        try {
            const realParentPath = await fs.realpath(parentDir);
            const normalizedParent = normalizePath(realParentPath);
            const isParentAllowed = allowedDirectories.some(dir => normalizedParent.startsWith(dir));
            if (!isParentAllowed) {
                throw new Error("访问被拒绝 - 父目录在允许的目录之外");
            }
            return absolute;
        }
        catch {
            throw new Error(`父目录不存在: ${parentDir}`);
        }
    }
}
// Schema定义
const ReadFileArgsSchema = z.object({
    path: z.string().describe('文件路径'),
});
const WriteFileArgsSchema = z.object({
    path: z.string().describe('文件路径'),
    content: z.string().describe('文件内容'),
});
const ListDirectoryArgsSchema = z.object({
    path: z.string().describe('目录路径'),
});
const SearchFilesArgsSchema = z.object({
    path: z.string().describe('搜索起始目录'),
    pattern: z.string().describe('搜索模式'),
    excludePatterns: z.array(z.string()).optional().default([]).describe('排除模式'),
});
// XLIFF特定的Schema
const ParseXliffArgsSchema = z.object({
    path: z.string().describe('XLIFF文件路径'),
});
const CreateXliffArgsSchema = z.object({
    path: z.string().describe('输出文件路径'),
    data: z.array(z.object({
        fileName: z.string(),
        segNumber: z.number(),
        percent: z.number(),
        source: z.string(),
        target: z.string(),
        srcLang: z.string(),
        tgtLang: z.string(),
        id: z.string().optional(),
        approved: z.boolean().optional(),
        state: z.string().optional(),
        resname: z.string().optional(),
    })).describe('XLIFF数据数组'),
    options: z.object({
        version: z.enum(['1.2', '2.0', '2.1']).optional().default('1.2'),
        sourceLanguage: z.string(),
        targetLanguage: z.string(),
        original: z.string().optional(),
        datatype: z.string().optional(),
    }).describe('创建选项'),
});
const UpdateXliffArgsSchema = z.object({
    path: z.string().describe('XLIFF文件路径'),
    updates: z.array(z.object({
        fileName: z.string(),
        segNumber: z.number(),
        percent: z.number(),
        source: z.string(),
        target: z.string(),
        srcLang: z.string(),
        tgtLang: z.string(),
        id: z.string().optional(),
        approved: z.boolean().optional(),
        state: z.string().optional(),
        resname: z.string().optional(),
    })).describe('更新数据数组'),
    options: z.object({
        updateMode: z.enum(['merge', 'replace']).default('merge'),
        preserveState: z.boolean().optional().default(false),
        preserveApproval: z.boolean().optional().default(false),
    }).describe('更新选项'),
});
const ValidateXliffArgsSchema = z.object({
    path: z.string().describe('XLIFF文件路径'),
});
const BatchProcessXliffArgsSchema = z.object({
    directory: z.string().describe('包含XLIFF文件的目录'),
    pattern: z.string().optional().default('*.xlf').describe('文件匹配模式'),
    operation: z.enum(['parse', 'validate']).describe('批处理操作类型'),
});
const ToolInputSchema = ToolSchema.shape.inputSchema;
// 服务器设置
const server = new Server({
    name: "xliff-mcp-server",
    version: "1.0.0",
}, {
    capabilities: {
        tools: {},
    },
});
// 工具实现
async function searchFiles(rootPath, pattern, excludePatterns = []) {
    const results = [];
    async function search(currentPath) {
        const entries = await fs.readdir(currentPath, { withFileTypes: true });
        for (const entry of entries) {
            const fullPath = path.join(currentPath, entry.name);
            try {
                await validatePath(fullPath);
                const relativePath = path.relative(rootPath, fullPath);
                const shouldExclude = excludePatterns.some(pattern => {
                    const globPattern = pattern.includes('*') ? pattern : `**/${pattern}/**`;
                    return minimatch(relativePath, globPattern, { dot: true });
                });
                if (shouldExclude) {
                    continue;
                }
                if (entry.name.toLowerCase().includes(pattern.toLowerCase())) {
                    results.push(fullPath);
                }
                if (entry.isDirectory()) {
                    await search(fullPath);
                }
            }
            catch (error) {
                continue;
            }
        }
    }
    await search(rootPath);
    return results;
}
// 工具处理程序
server.setRequestHandler(ListToolsRequestSchema, async () => {
    return {
        tools: [
            {
                name: "read_file",
                description: "读取文件的完整内容。只能在允许的目录中工作。",
                inputSchema: zodToJsonSchema(ReadFileArgsSchema),
            },
            {
                name: "write_file",
                description: "创建新文件或完全覆盖现有文件。只能在允许的目录中工作。",
                inputSchema: zodToJsonSchema(WriteFileArgsSchema),
            },
            {
                name: "list_directory",
                description: "列出指定路径中的所有文件和目录。只能在允许的目录中工作。",
                inputSchema: zodToJsonSchema(ListDirectoryArgsSchema),
            },
            {
                name: "search_files",
                description: "递归搜索匹配模式的文件和目录。只能在允许的目录中搜索。",
                inputSchema: zodToJsonSchema(SearchFilesArgsSchema),
            },
            {
                name: "parse_xliff",
                description: "解析XLIFF文件并返回JSON格式的翻译单元数据。支持XLIFF 1.2和2.x版本。",
                inputSchema: zodToJsonSchema(ParseXliffArgsSchema),
            },
            {
                name: "create_xliff",
                description: "根据JSON数据创建新的XLIFF文件。支持XLIFF 1.2和2.0格式。",
                inputSchema: zodToJsonSchema(CreateXliffArgsSchema),
            },
            {
                name: "update_xliff",
                description: "更新现有XLIFF文件的翻译内容。支持合并和替换模式。",
                inputSchema: zodToJsonSchema(UpdateXliffArgsSchema),
            },
            {
                name: "validate_xliff",
                description: "验证XLIFF文件的格式和结构是否正确。",
                inputSchema: zodToJsonSchema(ValidateXliffArgsSchema),
            },
            {
                name: "batch_process_xliff",
                description: "批量处理目录中的XLIFF文件。支持批量解析和验证。",
                inputSchema: zodToJsonSchema(BatchProcessXliffArgsSchema),
            },
            {
                name: "list_allowed_directories",
                description: "返回此服务器允许访问的目录列表。",
                inputSchema: {
                    type: "object",
                    properties: {},
                    required: [],
                },
            },
        ],
    };
});
server.setRequestHandler(CallToolRequestSchema, async (request) => {
    try {
        const { name, arguments: args } = request.params;
        switch (name) {
            case "read_file": {
                const parsed = ReadFileArgsSchema.safeParse(args);
                if (!parsed.success) {
                    throw new Error(`read_file参数无效: ${parsed.error}`);
                }
                const validPath = await validatePath(parsed.data.path);
                const content = await fs.readFile(validPath, "utf-8");
                return {
                    content: [{ type: "text", text: content }],
                };
            }
            case "write_file": {
                const parsed = WriteFileArgsSchema.safeParse(args);
                if (!parsed.success) {
                    throw new Error(`write_file参数无效: ${parsed.error}`);
                }
                const validPath = await validatePath(parsed.data.path);
                await fs.writeFile(validPath, parsed.data.content, "utf-8");
                return {
                    content: [{ type: "text", text: `成功写入文件 ${parsed.data.path}` }],
                };
            }
            case "list_directory": {
                const parsed = ListDirectoryArgsSchema.safeParse(args);
                if (!parsed.success) {
                    throw new Error(`list_directory参数无效: ${parsed.error}`);
                }
                const validPath = await validatePath(parsed.data.path);
                const entries = await fs.readdir(validPath, { withFileTypes: true });
                const formatted = entries
                    .map((entry) => `${entry.isDirectory() ? "[目录]" : "[文件]"} ${entry.name}`)
                    .join("\n");
                return {
                    content: [{ type: "text", text: formatted }],
                };
            }
            case "search_files": {
                const parsed = SearchFilesArgsSchema.safeParse(args);
                if (!parsed.success) {
                    throw new Error(`search_files参数无效: ${parsed.error}`);
                }
                const validPath = await validatePath(parsed.data.path);
                const results = await searchFiles(validPath, parsed.data.pattern, parsed.data.excludePatterns);
                return {
                    content: [{ type: "text", text: results.length > 0 ? results.join("\n") : "未找到匹配项" }],
                };
            }
            case "parse_xliff": {
                const parsed = ParseXliffArgsSchema.safeParse(args);
                if (!parsed.success) {
                    throw new Error(`parse_xliff参数无效: ${parsed.error}`);
                }
                const validPath = await validatePath(parsed.data.path);
                const content = await fs.readFile(validPath, "utf-8");
                const fileName = path.basename(validPath);
                const xliffData = XliffProcessor.parseXliff(fileName, content);
                return {
                    content: [{ type: "text", text: JSON.stringify(xliffData, null, 2) }],
                };
            }
            case "create_xliff": {
                const parsed = CreateXliffArgsSchema.safeParse(args);
                if (!parsed.success) {
                    throw new Error(`create_xliff参数无效: ${parsed.error}`);
                }
                const validPath = await validatePath(parsed.data.path);
                const xliffContent = XliffProcessor.createXliff(parsed.data.data, parsed.data.options);
                await fs.writeFile(validPath, xliffContent, "utf-8");
                return {
                    content: [{ type: "text", text: `成功创建XLIFF文件 ${parsed.data.path}` }],
                };
            }
            case "update_xliff": {
                const parsed = UpdateXliffArgsSchema.safeParse(args);
                if (!parsed.success) {
                    throw new Error(`update_xliff参数无效: ${parsed.error}`);
                }
                const validPath = await validatePath(parsed.data.path);
                const originalContent = await fs.readFile(validPath, "utf-8");
                const updatedContent = XliffProcessor.updateXliff(originalContent, parsed.data.updates, parsed.data.options);
                await fs.writeFile(validPath, updatedContent, "utf-8");
                return {
                    content: [{ type: "text", text: `成功更新XLIFF文件 ${parsed.data.path}` }],
                };
            }
            case "validate_xliff": {
                const parsed = ValidateXliffArgsSchema.safeParse(args);
                if (!parsed.success) {
                    throw new Error(`validate_xliff参数无效: ${parsed.error}`);
                }
                const validPath = await validatePath(parsed.data.path);
                const content = await fs.readFile(validPath, "utf-8");
                const validation = XliffProcessor.validateXliff(content);
                const result = {
                    valid: validation.valid,
                    errors: validation.errors,
                    file: parsed.data.path
                };
                return {
                    content: [{ type: "text", text: JSON.stringify(result, null, 2) }],
                };
            }
            case "batch_process_xliff": {
                const parsed = BatchProcessXliffArgsSchema.safeParse(args);
                if (!parsed.success) {
                    throw new Error(`batch_process_xliff参数无效: ${parsed.error}`);
                }
                const validPath = await validatePath(parsed.data.directory);
                const xliffFiles = await searchFiles(validPath, parsed.data.pattern);
                const results = [];
                for (const filePath of xliffFiles) {
                    try {
                        const content = await fs.readFile(filePath, "utf-8");
                        const fileName = path.basename(filePath);
                        if (parsed.data.operation === 'parse') {
                            const xliffData = XliffProcessor.parseXliff(fileName, content);
                            results.push({
                                file: filePath,
                                success: true,
                                data: xliffData
                            });
                        }
                        else if (parsed.data.operation === 'validate') {
                            const validation = XliffProcessor.validateXliff(content);
                            results.push({
                                file: filePath,
                                success: validation.valid,
                                valid: validation.valid,
                                errors: validation.errors
                            });
                        }
                    }
                    catch (error) {
                        results.push({
                            file: filePath,
                            success: false,
                            error: error instanceof Error ? error.message : String(error)
                        });
                    }
                }
                return {
                    content: [{ type: "text", text: JSON.stringify(results, null, 2) }],
                };
            }
            case "list_allowed_directories": {
                return {
                    content: [{
                            type: "text",
                            text: `允许访问的目录:\n${allowedDirectories.join('\n')}`
                        }],
                };
            }
            default:
                throw new Error(`未知工具: ${name}`);
        }
    }
    catch (error) {
        const errorMessage = error instanceof Error ? error.message : String(error);
        return {
            content: [{ type: "text", text: `错误: ${errorMessage}` }],
            isError: true,
        };
    }
});
// 启动服务器
async function runServer() {
    const transport = new StdioServerTransport();
    await server.connect(transport);
    console.error("XLIFF MCP服务器在stdio上运行");
    console.error("允许的目录:", allowedDirectories);
}
runServer().catch((error) => {
    console.error("服务器运行出现致命错误:", error);
    process.exit(1);
});
